@@ -16,12 +16,11 @@ import colorsys
 import numpy as np
 from skimage.measure import find_contours
 import matplotlib.pyplot as plt
-from matplotlib import patches,  lines
+from matplotlib import patches, lines
 from matplotlib.patches import Polygon
 import IPython.display
 
 
-
 # Import Mask RCNN
 from . import utils
 
@@ -30,8 +29,10 @@ from . import utils
 #  Visualization
 ############################################################
 
-def display_images(images, titles=None, cols=4, cmap=None, norm=None,
-                   interpolation=None):
+
+def display_images(
+    images, titles=None, cols=4, cmap=None, norm=None, interpolation=None
+):
     """Display the given set of images, optionally with titles.
     images: list or array of image tensors in HWC format.
     titles: optional. A list of titles to display with each image.
@@ -47,9 +48,10 @@ def display_images(images, titles=None, cols=4, cmap=None, norm=None,
     for image, title in zip(images, titles):
         plt.subplot(rows, cols, i)
         plt.title(title, fontsize=9)
-        plt.axis('off')
-        plt.imshow(image.astype(np.uint8), cmap=cmap,
-                   norm=norm, interpolation=interpolation)
+        plt.axis("off")
+        plt.imshow(
+            image.astype(np.uint8), cmap=cmap, norm=norm, interpolation=interpolation
+        )
         i += 1
     plt.show()
 
@@ -68,21 +70,31 @@ def random_colors(N, bright=True):
 
 
 def apply_mask(image, mask, color, alpha=0.5):
-    """Apply the given mask to the image.
-    """
+    """Apply the given mask to the image."""
     for c in range(3):
-        image[:, :, c] = np.where(mask == 1,
-                                  image[:, :, c] *
-                                  (1 - alpha) + alpha * color[c] * 255,
-                                  image[:, :, c])
+        image[:, :, c] = np.where(
+            mask == 1,
+            image[:, :, c] * (1 - alpha) + alpha * color[c] * 255,
+            image[:, :, c],
+        )
     return image
 
 
-def display_instances(image, boxes, masks, class_ids, class_names,
-                      scores=None, title="",
-                      figsize=(16, 16), ax=None,
-                      show_mask=True, show_bbox=True,
-                      colors=None, captions=None):
+def display_instances(
+    image,
+    boxes,
+    masks,
+    class_ids,
+    class_names,
+    scores=None,
+    title="",
+    figsize=(16, 16),
+    ax=None,
+    show_mask=True,
+    show_bbox=True,
+    colors=None,
+    captions=None,
+):
     """
     boxes: [num_instance, (y1, x1, y2, x2, class_id)] in image coordinates.
     masks: [height, width, num_instances]
@@ -115,7 +127,7 @@ def display_instances(image, boxes, masks, class_ids, class_names,
     height, width = image.shape[:2]
     ax.set_ylim(height + 10, -10)
     ax.set_xlim(-10, width + 10)
-    ax.axis('off')
+    ax.axis("off")
     ax.set_title(title)
 
     masked_image = image.astype(np.uint32).copy()
@@ -128,9 +140,16 @@ def display_instances(image, boxes, masks, class_ids, class_names,
             continue
         y1, x1, y2, x2 = boxes[i]
         if show_bbox:
-            p = patches.Rectangle((x1, y1), x2 - x1, y2 - y1, linewidth=2,
-                                alpha=0.7, linestyle="dashed",
-                                edgecolor=color, facecolor='none')
+            p = patches.Rectangle(
+                (x1, y1),
+                x2 - x1,
+                y2 - y1,
+                linewidth=2,
+                alpha=0.7,
+                linestyle="dashed",
+                edgecolor=color,
+                facecolor="none",
+            )
             ax.add_patch(p)
 
         # Label
@@ -141,8 +160,7 @@ def display_instances(image, boxes, masks, class_ids, class_names,
             caption = "{} {:.3f}".format(label, score) if score else label
         else:
             caption = captions[i]
-        ax.text(x1, y1 + 8, caption,
-                color='w', size=11, backgroundcolor="none")
+        ax.text(x1, y1 + 8, caption, color="w", size=11, backgroundcolor="none")
 
         # Mask
         mask = masks[:, :, i]
@@ -151,8 +169,7 @@ def display_instances(image, boxes, masks, class_ids, class_names,
 
         # Mask Polygon
         # Pad to ensure proper polygons for masks that touch image edges.
-        padded_mask = np.zeros(
-            (mask.shape[0] + 2, mask.shape[1] + 2), dtype=np.uint8)
+        padded_mask = np.zeros((mask.shape[0] + 2, mask.shape[1] + 2), dtype=np.uint8)
         padded_mask[1:-1, 1:-1] = mask
         contours = find_contours(padded_mask, 0.5)
         for verts in contours:
@@ -165,42 +182,74 @@ def display_instances(image, boxes, masks, class_ids, class_names,
         plt.show()
 
 
-def display_differences(image,
-                        gt_box, gt_class_id, gt_mask,
-                        pred_box, pred_class_id, pred_score, pred_mask,
-                        class_names, title="", ax=None,
-                        show_mask=True, show_box=True,
-                        iou_threshold=0.5, score_threshold=0.5):
+def display_differences(
+    image,
+    gt_box,
+    gt_class_id,
+    gt_mask,
+    pred_box,
+    pred_class_id,
+    pred_score,
+    pred_mask,
+    class_names,
+    title="",
+    ax=None,
+    show_mask=True,
+    show_box=True,
+    iou_threshold=0.5,
+    score_threshold=0.5,
+):
     """Display ground truth and prediction instances on the same image."""
     # Match predictions to ground truth
     gt_match, pred_match, overlaps = utils.compute_matches(
-        gt_box, gt_class_id, gt_mask,
-        pred_box, pred_class_id, pred_score, pred_mask,
-        iou_threshold=iou_threshold, score_threshold=score_threshold)
+        gt_box,
+        gt_class_id,
+        gt_mask,
+        pred_box,
+        pred_class_id,
+        pred_score,
+        pred_mask,
+        iou_threshold=iou_threshold,
+        score_threshold=score_threshold,
+    )
     # Ground truth = green. Predictions = red
-    colors = [(0, 1, 0, .8)] * len(gt_match)\
-           + [(1, 0, 0, 1)] * len(pred_match)
+    colors = [(0, 1, 0, 0.8)] * len(gt_match) + [(1, 0, 0, 1)] * len(pred_match)
     # Concatenate GT and predictions
     class_ids = np.concatenate([gt_class_id, pred_class_id])
     scores = np.concatenate([np.zeros([len(gt_match)]), pred_score])
     boxes = np.concatenate([gt_box, pred_box])
     masks = np.concatenate([gt_mask, pred_mask], axis=-1)
     # Captions per instance show score/IoU
-    captions = ["" for m in gt_match] + ["{:.2f} / {:.2f}".format(
-        pred_score[i],
-        (overlaps[i, int(pred_match[i])]
-            if pred_match[i] > -1 else overlaps[i].max()))
-            for i in range(len(pred_match))]
+    captions = ["" for m in gt_match] + [
+        "{:.2f} / {:.2f}".format(
+            pred_score[i],
+            (
+                overlaps[i, int(pred_match[i])]
+                if pred_match[i] > -1
+                else overlaps[i].max()
+            ),
+        )
+        for i in range(len(pred_match))
+    ]
     # Set title if not provided
-    title = title or "Ground Truth and Detections\n GT=green, pred=red, captions: score/IoU"
+    title = (
+        title or "Ground Truth and Detections\n GT=green, pred=red, captions: score/IoU"
+    )
     # Display
     display_instances(
         image,
-        boxes, masks, class_ids,
-        class_names, scores, ax=ax,
-        show_bbox=show_box, show_mask=show_mask,
-        colors=colors, captions=captions,
-        title=title)
+        boxes,
+        masks,
+        class_ids,
+        class_names,
+        scores,
+        ax=ax,
+        show_bbox=show_box,
+        show_mask=show_mask,
+        colors=colors,
+        captions=captions,
+        title=title,
+    )
 
 
 def draw_rois(image, rois, refined_rois, mask, class_ids, class_names, limit=10):
@@ -212,47 +261,62 @@ def draw_rois(image, rois, refined_rois, mask, class_ids, class_names, limit=10)
 
     # Pick random anchors in case there are too many.
     ids = np.arange(rois.shape[0], dtype=np.int32)
-    ids = np.random.choice(
-        ids, limit, replace=False) if ids.shape[0] > limit else ids
+    ids = np.random.choice(ids, limit, replace=False) if ids.shape[0] > limit else ids
 
     fig, ax = plt.subplots(1, figsize=(12, 12))
     if rois.shape[0] > limit:
-        plt.title("Showing {} random ROIs out of {}".format(
-            len(ids), rois.shape[0]))
+        plt.title("Showing {} random ROIs out of {}".format(len(ids), rois.shape[0]))
     else:
         plt.title("{} ROIs".format(len(ids)))
 
     # Show area outside image boundaries.
     ax.set_ylim(image.shape[0] + 20, -20)
     ax.set_xlim(-50, image.shape[1] + 20)
-    ax.axis('off')
+    ax.axis("off")
 
     for i, id in enumerate(ids):
         color = np.random.rand(3)
         class_id = class_ids[id]
         # ROI
         y1, x1, y2, x2 = rois[id]
-        p = patches.Rectangle((x1, y1), x2 - x1, y2 - y1, linewidth=2,
-                              edgecolor=color if class_id else "gray",
-                              facecolor='none', linestyle="dashed")
+        p = patches.Rectangle(
+            (x1, y1),
+            x2 - x1,
+            y2 - y1,
+            linewidth=2,
+            edgecolor=color if class_id else "gray",
+            facecolor="none",
+            linestyle="dashed",
+        )
         ax.add_patch(p)
         # Refined ROI
         if class_id:
             ry1, rx1, ry2, rx2 = refined_rois[id]
-            p = patches.Rectangle((rx1, ry1), rx2 - rx1, ry2 - ry1, linewidth=2,
-                                  edgecolor=color, facecolor='none')
+            p = patches.Rectangle(
+                (rx1, ry1),
+                rx2 - rx1,
+                ry2 - ry1,
+                linewidth=2,
+                edgecolor=color,
+                facecolor="none",
+            )
             ax.add_patch(p)
             # Connect the top-left corners of the anchor and proposal for easy visualization
             ax.add_line(lines.Line2D([x1, rx1], [y1, ry1], color=color))
 
             # Label
             label = class_names[class_id]
-            ax.text(rx1, ry1 + 8, "{}".format(label),
-                    color='w', size=11, backgroundcolor="none")
+            ax.text(
+                rx1,
+                ry1 + 8,
+                "{}".format(label),
+                color="w",
+                size=11,
+                backgroundcolor="none",
+            )
 
             # Mask
-            m = utils.unmold_mask(mask[id], rois[id]
-                                  [:4].astype(np.int32), image.shape)
+            m = utils.unmold_mask(mask[id], rois[id][:4].astype(np.int32), image.shape)
             masked_image = apply_mask(masked_image, m, color)
 
     ax.imshow(masked_image)
@@ -260,8 +324,11 @@ def draw_rois(image, rois, refined_rois, mask, class_ids, class_names, limit=10)
     # Print stats
     print("Positive ROIs: ", class_ids[class_ids > 0].shape[0])
     print("Negative ROIs: ", class_ids[class_ids == 0].shape[0])
-    print("Positive Ratio: {:.2f}".format(
-        class_ids[class_ids > 0].shape[0] / class_ids.shape[0]))
+    print(
+        "Positive Ratio: {:.2f}".format(
+            class_ids[class_ids > 0].shape[0] / class_ids.shape[0]
+        )
+    )
 
 
 # TODO: Replace with matplotlib equivalent?
@@ -270,10 +337,10 @@ def draw_box(image, box, color):
     color: list of 3 int values for RGB.
     """
     y1, x1, y2, x2 = box
-    image[y1:y1 + 2, x1:x2] = color
-    image[y2:y2 + 2, x1:x2] = color
-    image[y1:y2, x1:x1 + 2] = color
-    image[y1:y2, x2:x2 + 2] = color
+    image[y1 : y1 + 2, x1:x2] = color
+    image[y2 : y2 + 2, x1:x2] = color
+    image[y1:y2, x1 : x1 + 2] = color
+    image[y1:y2, x2 : x2 + 2] = color
     return image
 
 
@@ -285,10 +352,16 @@ def display_top_masks(image, mask, class_ids, class_names, limit=4):
     titles.append("H x W={}x{}".format(image.shape[0], image.shape[1]))
     # Pick top prominent classes in this image
     unique_class_ids = np.unique(class_ids)
-    mask_area = [np.sum(mask[:, :, np.where(class_ids == i)[0]])
-                 for i in unique_class_ids]
-    top_ids = [v[0] for v in sorted(zip(unique_class_ids, mask_area),
-                                    key=lambda r: r[1], reverse=True) if v[1] > 0]
+    mask_area = [
+        np.sum(mask[:, :, np.where(class_ids == i)[0]]) for i in unique_class_ids
+    ]
+    top_ids = [
+        v[0]
+        for v in sorted(
+            zip(unique_class_ids, mask_area), key=lambda r: r[1], reverse=True
+        )
+        if v[1] > 0
+    ]
     # Generate images and titles
     for i in range(limit):
         class_id = top_ids[i] if i < len(top_ids) else -1
@@ -315,8 +388,9 @@ def plot_precision_recall(AP, precisions, recalls):
     _ = ax.plot(recalls, precisions)
 
 
-def plot_overlaps(gt_class_ids, pred_class_ids, pred_scores,
-                  overlaps, class_names, threshold=0.5):
+def plot_overlaps(
+    gt_class_ids, pred_class_ids, pred_scores, overlaps, class_names, threshold=0.5
+):
     """Draw a grid showing how ground truth objects are classified.
     gt_class_ids: [N] int. Ground truth class IDs
     pred_class_id: [N] int. Predicted class IDs
@@ -329,34 +403,57 @@ def plot_overlaps(gt_class_ids, pred_class_ids, pred_scores,
     pred_class_ids = pred_class_ids[pred_class_ids != 0]
 
     plt.figure(figsize=(12, 10))
-    plt.imshow(overlaps, interpolation='nearest', cmap=plt.cm.Blues)
-    plt.yticks(np.arange(len(pred_class_ids)),
-               ["{} ({:.2f})".format(class_names[int(id)], pred_scores[i])
-                for i, id in enumerate(pred_class_ids)])
-    plt.xticks(np.arange(len(gt_class_ids)),
-               [class_names[int(id)] for id in gt_class_ids], rotation=90)
+    plt.imshow(overlaps, interpolation="nearest", cmap=plt.cm.Blues)
+    plt.yticks(
+        np.arange(len(pred_class_ids)),
+        [
+            "{} ({:.2f})".format(class_names[int(id)], pred_scores[i])
+            for i, id in enumerate(pred_class_ids)
+        ],
+    )
+    plt.xticks(
+        np.arange(len(gt_class_ids)),
+        [class_names[int(id)] for id in gt_class_ids],
+        rotation=90,
+    )
 
-    thresh = overlaps.max() / 2.
-    for i, j in itertools.product(range(overlaps.shape[0]),
-                                  range(overlaps.shape[1])):
+    thresh = overlaps.max() / 2.0
+    for i, j in itertools.product(range(overlaps.shape[0]), range(overlaps.shape[1])):
         text = ""
         if overlaps[i, j] > threshold:
             text = "match" if gt_class_ids[j] == pred_class_ids[i] else "wrong"
-        color = ("white" if overlaps[i, j] > thresh
-                 else "black" if overlaps[i, j] > 0
-                 else "grey")
-        plt.text(j, i, "{:.3f}\n{}".format(overlaps[i, j], text),
-                 horizontalalignment="center", verticalalignment="center",
-                 fontsize=9, color=color)
+        color = (
+            "white"
+            if overlaps[i, j] > thresh
+            else "black"
+            if overlaps[i, j] > 0
+            else "grey"
+        )
+        plt.text(
+            j,
+            i,
+            "{:.3f}\n{}".format(overlaps[i, j], text),
+            horizontalalignment="center",
+            verticalalignment="center",
+            fontsize=9,
+            color=color,
+        )
 
     plt.tight_layout()
     plt.xlabel("Ground Truth")
     plt.ylabel("Predictions")
 
 
-def draw_boxes(image, boxes=None, refined_boxes=None,
-               masks=None, captions=None, visibilities=None,
-               title="", ax=None):
+def draw_boxes(
+    image,
+    boxes=None,
+    refined_boxes=None,
+    masks=None,
+    captions=None,
+    visibilities=None,
+    title="",
+    ax=None,
+):
     """Draw bounding boxes and segmentation masks with different
     customizations.
 
@@ -385,7 +482,7 @@ def draw_boxes(image, boxes=None, refined_boxes=None,
     margin = image.shape[0] // 10
     ax.set_ylim(image.shape[0] + margin, -margin)
     ax.set_xlim(-margin, image.shape[1] + margin)
-    ax.axis('off')
+    ax.axis("off")
 
     ax.set_title(title)
 
@@ -412,16 +509,29 @@ def draw_boxes(image, boxes=None, refined_boxes=None,
                 # Skip this instance. Has no bbox. Likely lost in cropping.
                 continue
             y1, x1, y2, x2 = boxes[i]
-            p = patches.Rectangle((x1, y1), x2 - x1, y2 - y1, linewidth=2,
-                                  alpha=alpha, linestyle=style,
-                                  edgecolor=color, facecolor='none')
+            p = patches.Rectangle(
+                (x1, y1),
+                x2 - x1,
+                y2 - y1,
+                linewidth=2,
+                alpha=alpha,
+                linestyle=style,
+                edgecolor=color,
+                facecolor="none",
+            )
             ax.add_patch(p)
 
         # Refined boxes
         if refined_boxes is not None and visibility > 0:
             ry1, rx1, ry2, rx2 = refined_boxes[i].astype(np.int32)
-            p = patches.Rectangle((rx1, ry1), rx2 - rx1, ry2 - ry1, linewidth=2,
-                                  edgecolor=color, facecolor='none')
+            p = patches.Rectangle(
+                (rx1, ry1),
+                rx2 - rx1,
+                ry2 - ry1,
+                linewidth=2,
+                edgecolor=color,
+                facecolor="none",
+            )
             ax.add_patch(p)
             # Connect the top-left corners of the anchor and proposal
             if boxes is not None:
@@ -433,10 +543,16 @@ def draw_boxes(image, boxes=None, refined_boxes=None,
             # If there are refined boxes, display captions on them
             if refined_boxes is not None:
                 y1, x1, y2, x2 = ry1, rx1, ry2, rx2
-            ax.text(x1, y1, caption, size=11, verticalalignment='top',
-                    color='w', backgroundcolor="none",
-                    bbox={'facecolor': color, 'alpha': 0.5,
-                          'pad': 2, 'edgecolor': 'none'})
+            ax.text(
+                x1,
+                y1,
+                caption,
+                size=11,
+                verticalalignment="top",
+                color="w",
+                backgroundcolor="none",
+                bbox={"facecolor": color, "alpha": 0.5, "pad": 2, "edgecolor": "none"},
+            )
 
         # Masks
         if masks is not None:
@@ -445,7 +561,8 @@ def draw_boxes(image, boxes=None, refined_boxes=None,
             # Mask Polygon
             # Pad to ensure proper polygons for masks that touch image edges.
             padded_mask = np.zeros(
-                (mask.shape[0] + 2, mask.shape[1] + 2), dtype=np.uint8)
+                (mask.shape[0] + 2, mask.shape[1] + 2), dtype=np.uint8
+            )
             padded_mask[1:-1, 1:-1] = mask
             contours = find_contours(padded_mask, 0.5)
             for verts in contours:
@@ -476,23 +593,27 @@ def display_weight_stats(model):
     """
     layers = model.get_trainable_layers()
     table = [["WEIGHT NAME", "SHAPE", "MIN", "MAX", "STD"]]
-    for l in layers:
-        weight_values = l.get_weights()  # list of Numpy arrays
-        weight_tensors = l.weights  # list of TF tensors
+    for layer_ in layers:
+        weight_values = layer_.get_weights()  # list of Numpy arrays
+        weight_tensors = layer_.weights  # list of TF tensors
         for i, w in enumerate(weight_values):
             weight_name = weight_tensors[i].name
             # Detect problematic layers. Exclude biases of conv layers.
             alert = ""
-            if w.min() == w.max() and not (l.__class__.__name__ == "Conv2D" and i == 1):
+            if w.min() == w.max() and not (
+                layer_.__class__.__name__ == "Conv2D" and i == 1
+            ):
                 alert += "<span style='color:red'>*** dead?</span>"
             if np.abs(w.min()) > 1000 or np.abs(w.max()) > 1000:
                 alert += "<span style='color:red'>*** Overflow?</span>"
             # Add row
-            table.append([
-                weight_name + alert,
-                str(w.shape),
-                "{:+9.4f}".format(w.min()),
-                "{:+10.4f}".format(w.max()),
-                "{:+9.4f}".format(w.std()),
-            ])
+            table.append(
+                [
+                    weight_name + alert,
+                    str(w.shape),
+                    "{:+9.4f}".format(w.min()),
+                    "{:+10.4f}".format(w.max()),
+                    "{:+9.4f}".format(w.std()),
+                ]
+            )
     display_table(table)
